package org.one.stone.soup.mjdb.data.field;

import java.util.*;
/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The ArrayField class is a abstract wrapper for an array of
	* data.
	* @author Nik Cross
*/
public abstract class ArrayField extends Field {
	public static String DEFAULT_ID = "_Array";

	private boolean hashed = false;

	private Vector vector = new Vector();
	private Hashtable vectorHash = null;
	private Vector owners = new Vector();

public void _add(Field object,Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	vector.addElement(object);
	if(isHashed())
	{
		vectorHash.put(object.getId(),object);
	}
	owners.addElement(source);
	setChanged();

	if(report)
		reportChange(source);
}
public Field _get(int index) {
	return (Field)vector.elementAt(index);
}
public void _insert(Field object,int index,Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	vector.insertElementAt(object,index);
	if(isHashed())
	{
		vectorHash.put(object.getId(),object);
	}
	owners.insertElementAt(source,index);
	setChanged();

	if(report)
		reportChange(source);
}
public void _remove(int index,Object source) throws LockException{

	if(locked)
		throw new LockException(this+"is locked");

	if(isHashed())
	{
		vectorHash.remove(((Field)vector.elementAt(index)).getId());
	}
	vector.removeElementAt(index);
	owners.removeElementAt(index);
	setChanged();

	if(report)
		reportChange(source);
}
public void _remove(Field object,Object source) throws LockException{

	if(locked)
		throw new LockException(this+"is locked");

	if(isHashed())
	{
		vectorHash.remove(object.getId());
	}
	owners.removeElementAt(vector.indexOf(object));
	vector.removeElement(object);
	setChanged();

	if(report)
		reportChange(source);
}
public void _set(Field object,int index,Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	if(isHashed())
	{
		vectorHash.remove(((Field)vector.elementAt(index)).getId());
		vectorHash.put(object.getId(),object);
	}
	vector.setElementAt(object,index);
	owners.setElementAt(source,index);
	setChanged();

	if(report)
		reportChange(source);
}
/**
 * compare method comment.
 */
public int compare(Field target) throws CompareException {
	return 0;
}
public Field findId(String id) {

	if(isHashed())
	{
		Object field = vectorHash.get(id);

		if(field==null)
			return null;
		else
			return (Field)field;
	}
	else
	{
		for(int loop=0;loop<size();loop++)
		{
			if(_get(loop).getId().equals(id))
				return _get(loop);
		}
		return null;
	}
}
public Object getOwner(int index) {
	return owners.elementAt(index);
}
public Object getOwner(Object data) {
	int index = vector.indexOf(data);
	return owners.elementAt(index);
}
/**
 *
 * @return java.lang.String
 * @param name java.lang.String
 */
public String getValue(String name) {
	return null;
}
/**
 *
 * @return boolean
 */
public boolean hasChanged() {
	boolean state = super.hasChanged();

	for(int loop=0;loop<size();loop++)
	{
		if(_get(loop).hasChanged())
			state=true;
	}

	if(state==true)
		setChanged();

	return state;
}
/**
 *
 * @return boolean
 */
public boolean isHashed() {
	if(hashed==true && vectorHash==null)
	{
			vectorHash = new Hashtable();
	}
	return hashed;
}
public void register(DataLinker linker) {
	for(int loop=0;loop<size();loop++)
	{
		if(linker!=_get(loop))
		{
			try{
				linker.add(_get(loop),this);
			}
			catch(LockException le){}//dummy catch
			_get(loop).register(linker);
		}
	}
}
public void replace(Field oldObj, Field newObj)
{
	for(int loop=0;loop<size();loop++)
	{
		if(_get(loop)==oldObj)
		{
			try{
				_set(newObj,loop,this);
			}
			catch(LockException le){}//dummy catch
		}
		_get(loop).replace(oldObj,newObj);
	}
}
public void resetChanged() {
	super.resetChanged();

	for(int loop=0;loop<size();loop++)
	{
		_get(loop).resetChanged();
	}
}
/**
 * setValue method comment.
 */
public void setEmpty() throws LockException
{
		if(locked)
			throw new LockException(this+"is locked");

	vector = new Vector();
	if(isHashed())
	{
		vectorHash = new Hashtable();
	}
	owners = new Vector();
}
/**
 *
 * @param newHashed boolean
 */
public void setHashed(boolean newHashed) {
	hashed = newHashed;

	vectorHash = new Hashtable();
}
/**
 * setValue method comment.
 */
public void setValue(String name, String value, Object source) throws LockException
{
		if(locked)
		throw new LockException(this+"is locked");
}
/**
 * Insert the method's description here.
 * Creation date: (23/07/00 11:07:12)
 * @param object org.one.stone.soup.igml.IGMLObject
 */
public int size() {
	return vector.size();
}
}
