package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* Classes that wish to be lockable must implement this interface.
	* It supplies method templates that keep track of the <code>LockOwner</code>.
	* @author Nik Cross
*/

public interface Lockable {
/**
 * Called when a <code>LockOwner</code> requests that the data is locked
 * even if it is locked by another object.
 *
 * @param owner		The <code>LockOwner</code> that is requesting the lock
 */

 void forceLock(LockOwner owner);
/**
 * Called when a <code>LockOwner</code> requests that the data is unlocked
 * even if it is locked by another object.
 *
 * @param owner		The <code>LockOwner</code> that is requesting the lock
 */

void forceUnlock(LockOwner owner);
/**
 * Returns the <code>LockOwner</code> that holds a lock on the object.
 */

LockOwner getLockOwner();
/**
 * Returns true if the object is locked.
 */
boolean isLocked();
/**
 * Called when a <code>LockOwner</code> requests that the data is locked.
 * Only the <code>LockOwner</code> can release the lock.
 *
 * @param owner		The <code>LockOwner</code> that is requesting the lock
 */

void lock(LockOwner owner) throws LockException;
/**
 * Called when a <code>LockOwner</code> requests that the data is unlocked.
 * Only the <code>LockOwner</code> can unlock the data.
 *
 * @param owner		The <code>LockOwner</code> that is requesting the lock
 */

 void unlock(LockOwner owner) throws LockException;
}
