package org.one.stone.soup.mjdb.data.persistence;

import java.util.*;
import java.io.*;
import java.net.*;

import org.one.stone.soup.mjdb.data.field.*;
import org.one.stone.soup.mjdb.data.group.*;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

public class CsvToTable implements Persistence{
//	private final static String EOL = new String("\r\n");
//	private final static byte EOL_BV = 10;
//	private final static int BUFFER_SIZE = 1024;

	private char separator = ',';
	private boolean hasColumnTitles=false;
	private Table table;
	private String data;

	private InputStream iStream;
	private BufferedReader bStream;




public CsvToTable(int source,String path,String fileName,char separator) {
	this(source,path,fileName,separator,true);
}
public CsvToTable(int source,String path,String fileName,char separator,boolean hasColumnTitles) {
	this(null,source,path,fileName,separator,hasColumnTitles);
}
public CsvToTable(InputStream newStream) throws PersistenceException {
	this(newStream,',',true);
}
public CsvToTable(InputStream newStream,char separator) throws PersistenceException {
	this(newStream,separator,true);
}
public CsvToTable(InputStream newStream,char separator,boolean hasColumnTitles) throws PersistenceException{
	this(null,newStream,separator,hasColumnTitles);
}
public CsvToTable(String path,String fileName) {
	this(path,fileName,',',true);
}
public CsvToTable(String path,String fileName,char separator) {
	this(path,fileName,separator,true);
}
public CsvToTable(String path,String fileName,char separator,boolean hasColumnTitles) {
	this(null,path,fileName,separator,hasColumnTitles);
}
public CsvToTable(Table target,int source,String path,String fileName,char separator,boolean hasColumnTitles) {

	table=target;
	this.separator = separator;
	this.hasColumnTitles = hasColumnTitles;

	try{
		if(source==FILE)
			iStream = new FileInputStream(path+fileName);
		else
			iStream = new URL(path+fileName).openStream();
	}
	catch(Exception e)
	{
		//System.out.println(e);
	}

	bStream = new BufferedReader(new InputStreamReader(iStream));

/*	try{
		while(bStream.ready()==false)
		{
			try{
				Thread.sleep(10);
			}
			catch(Exception e){}
		}
	}
	catch(Exception e)
	{
		System.out.println("Ready error: "+e);
	}*/

	try{
		readTable();
	}
	catch(LockException e){/*Dummy Catch*/}
	catch(SetException e){/*Dummy Catch*/}

	try{
		iStream.close();
	}
	catch(Exception e)
	{
		System.out.println(e);
	}
}
public CsvToTable(Table target,InputStream newStream,char separator,boolean hasColumnTitles) throws PersistenceException {

	table = target;
	this.separator = separator;
	this.hasColumnTitles = hasColumnTitles;

	iStream = newStream;
	bStream = new BufferedReader(new InputStreamReader(iStream));

/*	int count=0;
	try{
		while(bStream.ready()==false)
		{
			try{
				Thread.sleep(10);
			}
			catch(Exception e){}
			count++;
			if(count>1000)
			{
				throw new PersistenceException("Table load buffer timed out");
			}
		}
	}
	catch(IOException ie)
	{
		throw new PersistenceException("Error while loading table "+ie.getMessage());
	}*/


	try{
		readTable();
	}
	catch(LockException e){
		e.printStackTrace();
	}
	catch(SetException e){
		e.printStackTrace();
	}

	try{
		iStream.close();
	}
	catch(IOException ie){
		throw new PersistenceException("Error while loading table "+ie.getMessage());
	}

}
public CsvToTable(Table target,String path,String fileName,char separator,boolean hasColumnTitles) {
	this(target,FILE,path,fileName,separator,hasColumnTitles);
}
public Table getTable() throws LockException,SetException{
	return getTable(true);
}
public Table getTable(boolean wrap) throws LockException,SetException{
	if(wrap)
		table.autoWrap(this);
	else
		table.autoWrapAsStrings(this);
	return table;
}
private void readColumnTitles() throws LockException,SetException{

	Vector titles =new Vector();
	readLine();
//	System.out.println("Read title line "+data);
	String field = "";
	int columnCount=0;

	StringTokenizer tokenizer = new StringTokenizer(data,""+separator);

	while(tokenizer.hasMoreTokens())
	{
		field=tokenizer.nextToken();
		if(field==null)
			break;

		//System.out.println("Read title "+field);
		titles.addElement(field);
		columnCount++;

		if(data==null)
			break;
	}

	if(table==null)
		table = new Table(columnCount);
	else
		table.setNoOfColumns(columnCount,this);
	for(int loop=0;loop<columnCount;loop++)
	{
		table.setColumnTitle(loop,(String)(titles.elementAt(loop)),this);
	}
	//System.out.println("Read titles");

	return;
}
private String readLine()
{
	try{
		data = bStream.readLine();
	}
	catch(Exception e)
	{
		System.out.println("Error reading line: "+e);
	}
	//System.out.println("Read line "+data);

	return data;
}
private Row readRow() throws SetException,LockException{
	Row temp = new Row(table.getNoOfColumns());

	data = readLine();

	if(data==null || data.length()==0)
	{
		//System.out.println("A. data=null");
		return null;
	}

	String field = "";
	int columnCount=0;

	StringTokenizer tokenizer = new StringTokenizer(data,""+separator,true);

	while(tokenizer.hasMoreTokens())
	{
		field=tokenizer.nextToken();
		if(field==null)
			field="";
		if(field.equals(""+separator))
		{
			field="";
			//System.out.println(field+", ");
			temp.setField(columnCount,"",this);
			columnCount++;
			continue;
		}

		//System.out.println(field+", ");

		temp.setField(columnCount,field,this);
		columnCount++;

		if(tokenizer.hasMoreTokens())
			field=tokenizer.nextToken();

		if(data==null)
			break;
	}

	if(columnCount!=table.getNoOfColumns())
	{
		if(columnCount<table.getNoOfColumns())
		{
			while(columnCount<table.getNoOfColumns())
			{
				field="";
				temp.setField(columnCount,field,this);
				columnCount++;
			}
			return temp;
		}
		else
		{
			System.out.println("columnCount mismatch");
			return null;
		}
	}
	else
		return temp;
}
private void readTable() throws SetException,LockException{

	readColumnTitles();

	if(hasColumnTitles==false)
	{
		Row row = new Row(table.getNoOfColumns());
		for(int loop=0;loop<row.getNoFields();loop++)
		{
			row.setField(loop,table.getColumnTitle(loop),this);
		}

		table.addRow(row,this);
	}

	Row temp;

	while(true)
	{
		temp=readRow();
		//System.out.println("Row Read");
		if(temp==null)
			break;

		table.addRow(temp,this);
		//System.out.println("Row Added");
	}
}
}
