/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

package org.one.stone.soup.net;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.one.stone.soup.io.Connection;
import org.one.stone.soup.io.ConnectionListener;
import org.one.stone.soup.io.StreamConnection;
import org.one.stone.soup.util.TimeWatch;

public class SocketToSocketConnection implements Connection,ActionListener{

	private long timeoutTime;
	private TimeWatch timer; 

	private Socket socket1;
	private Socket socket2;

	private ConnectionListener listener;

	private String alias;

	private StreamConnection up;
	private StreamConnection down;

	public SocketToSocketConnection( Socket socket1,Socket socket2,String alias,ConnectionListener listener,long timeoutTimeMilliSeconds ) throws IOException
	{
		this.alias = alias;
		this.listener = listener;
		this.socket1 = socket1;
		this.socket2 = socket2;

		timeoutTime = timeoutTimeMilliSeconds;
		
		timer = new TimeWatch(1000);
		timer.addActionListener(this);
		timer.setActionCommand("checkAlive");
		timer.start();
		
		up = new StreamConnection(socket1.getInputStream(),socket2.getOutputStream(),alias+".up",-1);
		down = new StreamConnection(socket2.getInputStream(),socket1.getOutputStream(),alias+".down",-1);

		up.connect(listener,"Up Stream Connection for "+alias );
		down.connect(listener,"Down Stream Connection for "+alias );
	}

	public String getAlias()
	{
		return alias;
	}

	public long getDataSentSize()
	{
		return up.getDataSentSize();
	}

	public long getDataReceivedSize()
	{
		return down.getDataSentSize();
	}

	public void close()
	{
		try{
			socket1.close();
			socket2.close();
		}
		catch(Exception e){}		
		
		timer.stop();
		up.disconnect();
		down.disconnect();
	}

	public void actionPerformed(ActionEvent event)
	{
	    checkAlive();
	}
	
	public void checkAlive()
	{
		if(System.currentTimeMillis()-up.getLastWriteTime()>timeoutTime && System.currentTimeMillis()-down.getLastWriteTime()>timeoutTime)
		{
		    System.out.println("Closed Socket Connection from "+socket1.getInetAddress()+" to "+socket2.getInetAddress());
			listener.connectionClosed( this );
		}
	}
	
	public InputStream getInputStream() throws IOException
	{
		return socket1.getInputStream();
	}
	public OutputStream getOutputStream() throws IOException
	{
		return socket1.getOutputStream();
	}
}
