package org.one.stone.soup.screen.recorder;

import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.GZIPInputStream;

public class ScreenPlayer implements Runnable{

	private static final int ALPHA = 0xFF000000;

	private InputStream iStream;
	private ScreenPlayerListener listener;

	private MemoryImageSource mis = null;
	private Rectangle area;

	private int rawDataSize;
	private int[] rawData;
	private int[] newRawData;
	
	private long startTime;
	private long frameTime;
	
	private boolean running;
	private boolean paused;
	private boolean fastForward;

	public ScreenPlayer(InputStream iStream,ScreenPlayerListener listener)
	{
		this.iStream = iStream;
		this.listener = listener;
		
		try{
			int width = iStream.read();
			width = width << 8;
			width += iStream.read();
			
			int height = iStream.read();
			height = height << 8;
			height += iStream.read();
			
			area = new Rectangle(width,height);
			rawDataSize = width*height;
			
			//System.out.println("Width:"+width+" Height:"+height);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public void play()
	{
		fastForward=false;
		paused=false;
		
		if(running==false)
		{
			new Thread(this,"Screen Player").start();
		}
	}
	
	public void pause()
	{
		paused=true;
	}
	
	public void stop()
	{
		paused=false;
		running=false;
	}
	
	public void fastforward()
	{
		fastForward=true;
		paused=false;
	}
	
	public synchronized void run()
	{
		startTime = System.currentTimeMillis();
		long lastFrameTime = 0;
		
		running=true;
		while(running==true)
		{
			while(paused==true)
			{
				try{ Thread.sleep(50); }catch(Exception e){}					
				startTime += 50;
			}
			
			try{
				readFrame();
				listener.newFrame();
			}
			catch(IOException ioe)
			{
				//ioe.printStackTrace();
				listener.showNewImage(null);
				break;
			}
			
			if(fastForward==true)
			{
				startTime -= (frameTime-lastFrameTime);
			}
			else
			{
				while( System.currentTimeMillis()-startTime<frameTime && running )
				{
					try{ Thread.sleep(100); }catch(Exception e){}
				}
			}
			
			lastFrameTime=frameTime;
		}
		running = false;
		
		listener.playerStopped();
	}
	
	private void readFrame() throws IOException
	{
		if(rawData==null)
		{
			rawData = new int[rawDataSize];
			newRawData = new int[rawDataSize];
		}
		
		int result = unpack();
		if( result==0 )
		{
			return;
		}
		else if( result==-1 )
		{
			running=false;
			return;
		}
		
		rawData = newRawData;
		
		if(mis==null)
		{
			mis = new MemoryImageSource(area.width,area.height,rawData,0,area.width);
			mis.setAnimated(true);
			listener.showNewImage( Toolkit.getDefaultToolkit().createImage(mis) );
			return;
		}
		else
		{
			mis.newPixels(rawData,ColorModel.getRGBdefault(),0,area.width);
			return;
		}		
	}
	
	private int unpack() throws IOException
	{
		//try{
			int time = iStream.read();
			time = time << 8;
			time += iStream.read();
			time = time << 8;
			time += iStream.read();
			time = time << 8;
			time += iStream.read();
			
			frameTime = (long)time;
			//System.out.println("ft:"+frameTime);
			
			byte type = (byte)iStream.read();
			//System.out.println("Packed Code:"+type);
			
			if(type<=0)
			{
				return type;
			}
		/*}
		catch(Exception e)
		{
			e.printStackTrace();
		}*/		

		ByteArrayOutputStream bO = new ByteArrayOutputStream();		
		try{
			int zSize = iStream.read();
			zSize = zSize << 8;
			zSize += iStream.read();
			zSize = zSize << 8;
			zSize += iStream.read();
			zSize = zSize << 8;
			zSize += iStream.read();

			byte[] zData = new byte[zSize];
			int readCursor = 0;
			int sizeRead = 0;
			
			while(sizeRead>-1)
			{
				readCursor+=sizeRead;
				if(readCursor>=zSize)
				{
					break;
				}
				
				sizeRead = iStream.read(zData,readCursor,zSize-readCursor);			
			}

			ByteArrayInputStream bI = new ByteArrayInputStream(zData);		
			
			//ZipInputStream zI = new ZipInputStream(bI);
			//zI.getNextEntry();

			GZIPInputStream zI = new GZIPInputStream(bI);

			byte[] buffer = new byte[1000];
			sizeRead = zI.read(buffer);
			
			while(sizeRead>-1)
			{
				bO.write(buffer,0,sizeRead);
				bO.flush();
				
				sizeRead = zI.read(buffer);
			}
			bO.flush();
			bO.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return 0;
		}
		
		byte[] packed = bO.toByteArray();
		//System.out.println("UnZipped To: "+packed.length);
		
		return runLengthDecode(packed);
}

private int runLengthDecode(byte[] packed)
{				
		newRawData = new int[rawDataSize];
		
		int inCursor = 0;
		int outCursor = 0;
		
		int blockSize = 0;
		
		int rgb = 0xFF000000;
		
		while(inCursor<packed.length && outCursor<rawDataSize)
		{
			if(packed[inCursor]==-1)
			{
				inCursor++;
				
				int count = (packed[inCursor] & 0xFF);
				inCursor++;
				
				for(int loop=0;loop<(126*count);loop++)
				{
					newRawData[outCursor]=rawData[outCursor];
					//newRawData[outCursor]=blue;
					outCursor++;
					if(outCursor==newRawData.length)
					{
						break;
					}
				}
			}
			else if(packed[inCursor]<0) // uncomp
			{
				blockSize = packed[inCursor] & 0x7F;//(128+packed[inCursor]);
				inCursor++;
				
				for(int loop=0;loop<blockSize;loop++)
				{
					rgb = ((packed[inCursor] & 0xFF)<<16) | ((packed[inCursor+1] & 0xFF)<<8) | (packed[inCursor+2] & 0xFF) | ALPHA;
					if(rgb==ALPHA)
					{
						rgb=rawData[outCursor];
					}
					//rgb = green;
					inCursor+=3;
					newRawData[outCursor]=rgb;
					outCursor++;
					if(outCursor==newRawData.length)
					{
						break;
					}
				}
			}
			else
			{
				blockSize = packed[inCursor];
				inCursor++;
				rgb = ((packed[inCursor] & 0xFF)<<16) | ((packed[inCursor+1] & 0xFF)<<8) | (packed[inCursor+2] & 0xFF) | ALPHA;
				
				boolean transparent = false;
				if(rgb==ALPHA)
				{
					transparent = true;
				}
				//rgb = red;
				inCursor+=3;
				
				for(int loop=0;loop<blockSize;loop++)
				{
					if(transparent)
					{
						newRawData[outCursor]=rawData[outCursor];						
					}
					else
					{
						newRawData[outCursor]=rgb;	
					}
					outCursor++;
					if(outCursor==newRawData.length)
					{
						break;
					}
				}
			}
		}
		
		return outCursor;
	}
}
