package org.one.stone.soup.xapp.swing.components;

import java.awt.Component;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import org.one.stone.soup.xapp.XappComponentStore;
import org.one.stone.soup.xapp.XappReceiver;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.XappSender;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.XappMenu;
import org.one.stone.soup.xapp.components.XappMenuItem;
import org.one.stone.soup.xapp.containers.XappMenuBar;
import org.one.stone.soup.xapp.swing.factories.XappSwingComponentFactory;
import org.one.stone.soup.xml.*;

public class XappSwingMenuBar extends JMenuBar implements ActionListener,XappSender, XappMenuBar {
	private XappReceiver receiver;
	private XappComponentStore menuStore;
	private XappComponentStore componentStore;
/**
 * XMenuBar constructor comment.
 */
public XappSwingMenuBar(XmlElement xMenuBar,XappComponentStore componentStore) {
	super();

	this.componentStore = componentStore;
	init(xMenuBar);
}
public void actionPerformed(java.awt.event.ActionEvent e)
{
	XmlElement xui = new XmlElement("action");
	xui.addAttribute("name",getName());

	XmlElement item = xui.addChild(((JMenuItem)e.getSource()).getName());
	item.addValue(e.getActionCommand());

	receiver.receiveXappData( xui );
}
private void init(XmlElement xMenuBar)
{
	menuStore = new XappComponentStore();

	String name = xMenuBar.getAttributeValueByName("name");
	if(name!=null)
	{
		setName(name);
		menuStore.addXappComponent("menuBar."+name,this);
	}

	for(int loop=0;loop<xMenuBar.getElementCount();loop++)
	{
		XmlElement xMenu = xMenuBar.getElementByIndex(loop);

		addMenu(xMenu);
	}
}

/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#getMenu(java.lang.String)
 */
public JMenu getMenu(String name)
{
	for(int loop=0;loop<getMenuCount();loop++)
	{
		if(getMenu(loop)!=null && getMenu(loop).getName()!=null && getMenu(loop).getName().equals(name))
		{
			return getMenu(loop);
		}
	}
	return null;
}

/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#removeMenu(org.one.stone.soup.xml.XmlElement)
 */
public void removeMenu(XmlElement xMenu)
{
	if(!xMenu.getName().equals("menu"))
	{
		return;
	}

	String name = xMenu.getAttributeValueByName("name");
	JMenu menu = getMenu(name);
		
	for(int loop=0;loop<xMenu.getElementCount();loop++)
	{
		XmlElement xMenuItem = xMenu.getElementByIndex(loop);
		
		XappMenuItem item = (XappMenuItem)menuStore.getXappComponent("menuItem."+xMenuItem.getAttributeValueByName("name"));
		menu.remove((JMenuItem)item);
		
		menuStore.removeXappComponent("menuItem."+xMenuItem.getAttributeValueByName("name"));
		componentStore.removeXappComponent("menuItem."+xMenuItem.getAttributeValueByName("name"));
	}
	
	for(int loop=0;loop<menu.getItemCount();loop++)
	{
		if(menu.getItem(loop)==null)
		{
			menu.remove(loop);
		}
	}
	
	if(menu.getItemCount()==0)
	{
		remove(menu);

		menuStore.removeXappComponent("menu."+xMenu.getAttributeValueByName("name"));
		componentStore.removeXappComponent("menu."+xMenu.getAttributeValueByName("name"));
	}
	
	doLayout();
}

/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#addMenu(org.one.stone.soup.xml.XmlElement)
 */
public void addMenu(XmlElement xMenu)
{
	if(!xMenu.getName().equals("menu"))
	{
		return;
	}

	String name = xMenu.getAttributeValueByName("name");
	JMenu menu = getMenu(name);
	if(menu==null)
	{
		menu = new XappSwingMenu();
		menu.setName(name);
		menu.setName(xMenu.getAttributeValueByName("name"));
		menuStore.addXappComponent("menu."+xMenu.getAttributeValueByName("name"),(XappMenu)menu);
	}
	else
	{
		menu.addSeparator();
	}
	
	String command = xMenu.getAttributeValueByName("command");
	if(command!=null)
	{
		menu.setActionCommand(command);
	}
	else
	{
		menu.setActionCommand(xMenu.getAttributeValueByName("name"));
	}
	//menuStore.put(xMenu.getAttributeValueByName("name"),menu);
	menuStore.addXappComponent("menu."+xMenu.getAttributeValueByName("name"),(XappComponent)menu);

	String enabled = xMenu.getAttributeValueByName("enabled");
	if(enabled!=null && enabled.equals("false"))
	{
		menu.setEnabled(false);
	}

	XappRootApplication.getComponentFactory().buildMenu(xMenu,this,(XappMenu)menu,menuStore);

	menu.addActionListener( this );
	if(this.getComponentCount()>0 && this.getComponent(this.getComponentCount()-1).getName().equals("Help"))
	{
		Component help = this.getComponent(this.getComponentCount()-1);
		Component box = (Component)this.getComponent(this.getComponentCount()-2);
		this.remove(help);
		if(box instanceof Box.Filler)
		{
			this.remove(box);
		}
		add(menu);
		add(Box.createGlue());
		add(help);
	}
	else if(menu.getName().equals("Help"))
	{
		add(Box.createGlue());
		add(menu);
	}
	else
	{
		add(menu);
	}

	componentStore.addXappComponentStore(menuStore);	
}

/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#setEnabled(java.lang.String, boolean)
 */
public void setEnabled(String path, boolean state)
{
	JMenuItem item = (JMenuItem)menuStore.getXappComponent(path);

	if(item!=null)
	{
		item.setEnabled(state);
	}
}
/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#addReceiver(org.one.stone.soup.xapp.XappReceiver)
 */
public void addReceiver(XappReceiver receiver)
{
	this.receiver = receiver;
}

/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#removeReceiver(org.one.stone.soup.xapp.XappReceiver)
 */
public void removeReceiver(XappReceiver receiver)
{
	this.receiver = null;
}
/* (non-Javadoc)
 * @see org.one.stone.soup.xapp.containers.XappMenuBar#setVisible(java.lang.String, boolean)
 */
public void setVisible(String path, boolean state)
{
	JMenuItem item = (JMenuItem)menuStore.getXappComponent(path);
/*
	String parentName = path.substring(0,path.indexOf('.'));
	JMenu parent = (JMenu)menuStore.get(parentName);*/

	if(item!=null)
	{
/*		if(state==false)
		{
			parent.remove(item);
		}
		else
		{
			parent.add(item);
		}*/
		item.setVisible(state);
	}
}

public void refresh()
{
	/*try{
		doLayout();
	}
	catch(ArrayIndexOutOfBoundsException e){}*/
}
public void addActionListener(ActionListener actionListener) {
	// TODO Auto-generated method stub
	
}


public void setTip(String text)
{
	setToolTipText(text);
}
}
