package org.one.stone.soup.xml;

import java.io.*;

 /*
 * Wet-Wired.com Xml Version Date 25-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
  * The XmlImportProcessor class will process import compiler tags
  * in an xml document.
  * The file specified in an import tag will be inserted at this point in the
  * Xml document model. eg. &lt;?import url="http://www.mysite.com/myDocPart.xml"?&gt;
  * It is used by adding an instance to an instance of XmlParser
  * via the addCompiler method.
 */

public class XmlImportProcessor implements XmlProcessor {
/**
 * XmlImportCompiler constructor comment.
 */
public XmlImportProcessor() {
	super();
}
public boolean process(XmlElement root,XmlElement directive, XmlElement target,int lineNo,String currentFileName) throws XmlParseException {

	if( !directive.getName().equals("import"))
		return false;

	String fileName = directive.getAttributeValueByName("url");

	try{
		XmlParser parser =  new XmlParser();
		XmlElement newElement = null;
		
		if( fileName != null )
		{
			InputStream iStream = null;
			iStream = org.one.stone.soup.browser.Browser.getInputStream(fileName);
			parser.setCurrentFileName( org.one.stone.soup.browser.Browser.getURLFor(fileName) );
			newElement = parser.parseDocument(iStream);
			iStream.close();
		}
		else
		{
			fileName = directive.getAttributeValueByName("file");
			if(fileName==null)
			{
				throw new XmlParseException(
					XmlParseException.NO_FILE_TO_IMPORT,
					new Object[]{
						""+lineNo,
						"in file "+currentFileName
					}
				);
			}

			File file = new File( new File(currentFileName).getParentFile().getAbsolutePath()+"/"+fileName );
			newElement = parser.parseDocument(file);
		}

		// strip off document wrapper
		for(int loop2=0;loop2<newElement.getElementCount();loop2++)
		{
			if(newElement.dataTypeFor(loop2)==XmlElement.VALUE)
				target.addValue(newElement.getValue(loop2));
			else
				target.addChild(newElement.getElementByIndex(loop2));
		}
	}
	catch(IOException e)
	{
		throw new XmlParseException(
			XmlParseException.FAILED_TO_IMPORT_FILE,
			new Object[]{
				fileName,
				""+lineNo,
				e.toString()+" "+e.getMessage() + "in file" + currentFileName
			}
		);
	}

	return true;
}
}
