package org.one.stone.soup.xml.wrapper;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;

import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlHelper;

/*
 * Wet-Wired.com Xml Helpers Version Date 25-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */

public class XmlElementWrapper extends XmlElement {

	private XmlElement xContents;
	private XmlElement root;

	private int[] indexPath;
	private boolean created = false;
	private boolean building = false;

	public XmlElementWrapper(String name,XmlElement root,int[] indexPath)
	{
		super(name);
		this.root = root;
		this.indexPath = indexPath;
		created = false;
	}

	public XmlElementWrapper(XmlElement root,XmlElement hold)
	{
		super(hold.getName());
		this.root = root;
		xContents = hold;
		created = true;
	}

	public XmlElement getContents()
	{
		checkCreated();
		return xContents;
	}

	public String toXmlBeautify(boolean ignorSystemAttributes) {
		//checkCreated();
		return toXmlNoBeautify(ignorSystemAttributes)+"\n";
	}

	public void toXmlBeautify(OutputStream oStream,boolean ignorSystemAttributes) throws IOException
	{
		toXmlNoBeautify(oStream,ignorSystemAttributes);
	}

	public void toXmlNoBeautify(OutputStream oStream,boolean ignorSystemAttributes) throws IOException
	{
		checkCreated();
		if(root==null || xContents==null)
		{
			return;
		}
		String path = XmlHelper.getIndexPath(root,xContents);

		oStream.write( ("<?elementWrapper").getBytes() );

		/*for(int loop=0;loop<getAttributeCount();loop++)
		{
			String attributeName = getAttributeNameByIndex(loop);//(Attribute)attributes.elementAt(loop);

			if( attributeName==null || (ignorSystemAttributes==true && attributeName.charAt(0)=='_') )
				continue;

			String attributeValue = getAttributeValueByIndex(loop);//(Attribute)attributes.elementAt(loop);
			string+=" "+attributeName+"=\""+attributeValue+"\"";
		}*/

		for(int loop=0;loop<super.getAttributeCount();loop++)
		{
			String attributeName = super.getAttributeNameByIndex(loop);//(Attribute)attributes.elementAt(loop);

			if( attributeName==null || (ignorSystemAttributes==true && attributeName.charAt(0)=='_') )
				continue;

			String attributeValue = super.getAttributeValueByIndex(loop);//(Attribute)attributes.elementAt(loop);
			oStream.write( (" "+attributeName+"=\""+attributeValue+"\"").getBytes() );
		}

		oStream.write( (" name=\""+getName()+"\" path=\""+path+"\"?>").getBytes() );
	}

	private synchronized void checkCreated()
	{
		if(created==true)
		{
			return;
		}

		building = true;

		xContents = XmlHelper.getElementByIndexPath(root,indexPath);

		if(xContents==null)
		{
			System.out.println("XmlElementWrapper path "+indexPath+" not found.");
			return;
		}

		/*for(int loop=0;loop<xContents.getAttributeCount();loop++)
		{
			//if(getAttributeValueByName(xContents.getAttributeNameByIndex(loop))==null)
			//{
				addAttribute( xContents.getAttributeNameByIndex(loop),xContents.getAttributeValueByIndex(loop) );
			//}
		}*/

		building = false;
		created = true;
	}

	public int getAttributeCount()
	{
		checkCreated();
		return xContents.getAttributeCount();
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#getAttributeByIndex(int)
	 */
	public String getAttributeNameByIndex(int index) {
		if(building)
		{
			return null;
		}

		checkCreated();
		String attributeName = xContents.getAttributeNameByIndex(index);

		return attributeName;
	}

	public String getAttributeValueByIndex(int index) {
		if(building)
		{
			return null;
		}

		checkCreated();
		String attributeValue = xContents.getAttributeValueByIndex(index);

		return attributeValue;
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#getAttributeValueByName(java.lang.String)
	 */
	public String getAttributeValueByName(String name) {
		if(building)
		{
			return null;
		}

		checkCreated();

		if(super.getAttributeValueByName(name)!=null)
		{
			return super.getAttributeValueByName(name);
		}

		String attributeValue = xContents.getAttributeValueByName(name);
		return attributeValue;
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#getElementByIndex(int)
	 */
	public XmlElement getElementByIndex(int index) {
		//checkCreated();
		//return xContents.getElementByIndex(index);
		return null;
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#getElementByName(java.lang.String)
	 */
	public XmlElement getElementByName(String name) {
		//checkCreated();
		//return xContents.getElementByName(name);
		return null;
	}

	public int getElementCount()
	{
		return 0;
	}

	public Vector getElements()
	{
		return new Vector();
	}

	public void setCreated(boolean created)
	{
		this.created = created;
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#addAttribute(java.lang.String, java.lang.String)
	 */
	public void addAttribute(String name, String value) {
		checkCreated();
		super.addAttribute(name, value);
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xml.XmlElement#setAttributeValue(java.lang.String, java.lang.String)
	 */
	public void setAttributeValue(String name, String value) {
		checkCreated();
		super.setAttributeValue(name, value);
	}
}
